@@ -28,11 +28,10 @@
 
 package ORM;
 
-use 5.008001;
+use 5.006001;
 use strict;
 use warnings;
 use Carp;
-use Scalar::Util 'weaken';
 use base 'Class::Data::Inheritable';
 
 use ORM::Error;
@@ -58,8 +57,7 @@ ORM->mk_classdata( '_history_class' );
 ORM->mk_classdata( '_default_prefer_lazy_load' );
 ORM->mk_classdata( '_emulate_foreign_keys' );
 ORM->mk_classdata( '_default_cache_size' );
-
-my %ta;
+ORM->mk_classdata( '_current_transaction' );
 
 ##
 ## CONSTRUCTORS
@@ -96,6 +94,7 @@ sub new
     my $class   = shift;
     my %arg     = @_;
     my $error   = ORM::Error->new;
+    my $ta      = $class->new_transaction( error=>$error );
     my $self    = {};
     my $history = defined $arg{history} ? $arg{history} : $class->history_is_enabled;
 
@@ -110,6 +109,8 @@ sub new
 
         bless $self, $class;
 
+        $self->{_ORM_tpm} = 1 if( $arg{temporary} );
+
         # Extract required DB properties from %arg
         for $prop ( $class->_not_mandatory_props )
         {
@@ -130,7 +131,7 @@ sub new
     unless( $error->fatal )
     {
         # Check validity of object data
-        $self->_validate_prop( prop=>$self->{_ORM_data}, error=>$error );
+        $self->_validate_prop( prop=>$self->{_ORM_data}, method=>'new', error=>$error );
     }
 
     if( ! $arg{temporary} && ! $error->fatal )
@@ -150,11 +151,12 @@ sub new
         if( !$error->fatal && $history )
         {
             $class->_history_class->new( obj=>$self, created=>1, error=>$error );
-            $self->delete( error=>$error, history=>0 ) if( $error->fatal );
         }
+
+        # Cache object
+        $self->_cache->add( $self ) unless( $error->fatal );
     }
 
-    $self->_cache->add( $self ) unless( $error->fatal );
     $error->upto( $arg{error} );
     return $error->fatal ? undef : $self;
 }
@@ -299,28 +301,6 @@ sub find
 ##     error     => ORM::Error,
 ## );
 ##
-## In this context 'lazy_load' means that object will
-## not be loaded at all. Loading of object data
-## will be delayed until its properties will be demanded.
-##
-## find_id() with 'lazy_load' mostly useful when utilized
-## by ORM->stat().
-##
-##   lazy_load == undef,   
-##   ,    
-## prefer_lazy_load().
-##
-##   find_id()   'lazy_load'  
-##       
-##   ,     
-##  $class,     find_id().
-##        finish_loading()
-##     ,     
-##         $class.
-##
-## finish_loading()     update()   
-## ,    .
-##
 sub find_id
 {
     my $class = shift;
@@ -427,21 +407,8 @@ sub new_transaction
     my $class  = shift;
     my $iclass = $class->initial_class;
     my %arg    = @_;
-    my $my_ta;
 
-    if( $ta{$iclass} )
-    {
-        $my_ta = ORM::Ta->new;
-    }
-    else
-    {
-        $my_ta       = ORM::Ta->new( db=>$class->_db, error=>$arg{error} );
-        $ta{$iclass} = $my_ta;
-
-        weaken $ta{$iclass};
-    }
-
-    return $my_ta;
+    ORM::Ta->new( class=>$iclass, error=>$arg{error} );
 }
 
 ## use: $self->update
@@ -458,6 +425,7 @@ sub update
     my $class    = ref $self;
     my %arg      = @_;
     my $error    = ORM::Error->new;
+    my $ta       = $class->new_transaction( error=>$error );
     my $history  = defined $arg{history} ? $arg{history} : $class->history_is_enabled;
     my %changed_prop;
     my %expr_prop;
@@ -525,9 +493,9 @@ sub update
         }
     }
     # User validations
-    unless( $error->fatal )
+    if( %changed_prop && !$error->fatal )
     {
-        $self->_validate_prop( prop=>\%changed_prop, error=>$error );
+        $self->_validate_prop( prop=>\%changed_prop, old=>\%old_prop, method=>'update', error=>$error );
     }
     # Detect data changes again to consider changes in _validate_prop
     unless( $error->fatal )
@@ -547,7 +515,7 @@ sub update
         }
     }
 
-    if( $self->id && !$error->fatal && scalar( %changed_prop ) )
+    if( !$self->is_temporary && !$error->fatal && scalar( %changed_prop ) )
     {
         for my $prop ( keys %expr_prop )
         {
@@ -609,9 +577,10 @@ sub delete
     my $class   = ref $self;
     my %arg     = @_;
     my $error   = ORM::Error->new;
+    my $ta      = $class->new_transaction( error=>$error );
     my $history = defined $arg{history} ? $arg{history} : $class->history_is_enabled;
 
-    if( $self->id )
+    unless( $self->is_temporary )
     {
         unless( $error->fatal )
         {
@@ -698,7 +667,7 @@ sub finish_loading
         my $error = ORM::Error->new;
         my $data  = $class->_db->select_tables
         (
-            id     => $self->id,
+            id     => $self->qc( $self->id ),
             tables => $self->{_ORM_missing_tables},
             error  => $error,
         );
@@ -742,17 +711,25 @@ sub finish_loading
     if( $new_class && $new_class ne $class )
     {
         $class->_load_ORM_class( $new_class );
-        bless $self, $new_class;
 
-        my $base_class_tables = $class->_db_tables_count;
-        my $class_tables      = $new_class->_db_tables_count;
-
-        for( my $i=$base_class_tables; $i<$class_tables; $i++ )
+        if( UNIVERSAL::isa( $new_class, $class ) )
         {
-            $self->{_ORM_missing_tables}{$new_class->_db_table($i)} = 1;
-        }
+            bless $self, $new_class;
 
-        $self->finish_loading( error=>$arg{error} ) unless( defined $prop );
+            my $base_class_tables = $class->_db_tables_count;
+            my $class_tables      = $new_class->_db_tables_count;
+
+            for( my $i=$base_class_tables; $i<$class_tables; $i++ )
+            {
+                $self->{_ORM_missing_tables}{$new_class->_db_table($i)} = 1;
+            }
+
+            $self->finish_loading( error=>$arg{error} ) unless( defined $prop );
+        }
+        else
+        {
+            $self->_rebless_to_broken( deleted=>1 );
+        }
     }
 }
 
@@ -762,7 +739,7 @@ sub finish_loading
 
 sub id           { $_[0]->{_ORM_data}{id}; }
 sub class        { ref $_[0] || $_[0]; }
-sub is_temporary { ! $_[0]->{_ORM_data}{id}; }
+sub is_temporary { $_[0]->{_ORM_tpm}; }
 
 sub __ORM_db_value { $_[0]->{_ORM_data}{id}; }
 sub __ORM_new_db_value
@@ -975,11 +952,6 @@ sub _has_rev_ref
 ##     return_res  => boolean,
 ## )
 ##
-##  'count' ,   
-##        
-##  . page  pagesize   
-##   count.
-##
 sub stat
 {
     my $class    = shift;
@@ -1073,6 +1045,7 @@ sub stat
         );
     }
 
+    # Final step, prepare resulting data
     if( $res && !$error->fatal )
     {
         if( $arg{count} )
@@ -1125,6 +1098,7 @@ sub stat
 ##            either object referenced by id in DB,
 ##            or object referenced by value in DB
 ##
+sub _prop { shift->_property( @_ ); }
 sub _property
 {
     my $self   = shift;
@@ -1191,11 +1165,13 @@ sub _property
 ##            either object referenced by id in DB,
 ##            or object referenced by value in DB
 ##
+sub _prop_id { shift->_property_id( @_ ); }
 sub _property_id
 {
     my $self   = shift;
     my %arg;
     my $prop;
+    my $value;
 
     if( @_ == 1 )
     {
@@ -1207,14 +1183,23 @@ sub _property_id
         $prop = $arg{name};
     }
 
-    if( exists $self->{_ORM_missing_tables} )
+    if( $prop eq 'class' )
     {
-        $self->finish_loading( prop=>$prop, error=>$arg{error} )
+        $value = $self->class;
+    }
+    else
+    {
+        if( exists $self->{_ORM_missing_tables} )
+        {
+            $self->finish_loading( prop=>$prop, error=>$arg{error} );
+        }
+        $value = $self->{_ORM_data}{$prop};
     }
 
-    return $self->{_ORM_data}{$prop};
+    return $value;
 }
 
+sub _rev { shift->_rev_prop( @_ ); }
 sub _rev_prop
 {
     my $self      = shift;
@@ -1229,6 +1214,7 @@ sub _rev_prop
     }
 }
 
+sub _rev_count { shift->_rev_prop_count( @_ ); }
 sub _rev_prop_count
 {
     my $self      = shift;
@@ -1258,7 +1244,7 @@ sub AUTOLOAD
         my $self = shift;
         my %arg  = @_;
 
-        die "Called undefined static method '$ORM::AUTOLOAD' of class '$self'" unless( ref $self );
+        croak "Called undefined static method '$ORM::AUTOLOAD' of class '$self'" unless( ref $self );
 
         $self->_property( name=>$prop, %arg );
     }
@@ -1332,6 +1318,7 @@ sub _rebless_to_broken
         $self->{error} = $arg{error};
     }
 
+    delete $self->{_ORM_tmp};
     delete $self->{_ORM_data};
     delete $self->{_ORM_cache};
     delete $self->{_ORM_missing_tables};
@@ -1341,10 +1328,8 @@ sub _rebless_to_broken
 
 ## use: $self->_normalize_prop_to_db_value( name=>STRING, value=>SCALAR, error=>ORM::Error )
 ##
-##    ()  
-##        .
-##
-##   .
+## Normalize specified value to be able to store it in database table.
+## All arguments are necessary.
 ##
 sub _normalize_prop_to_db_value
 {
@@ -1420,7 +1405,7 @@ sub _normalize_prop_to_db_value
                 value => $prop_value,
                 error => $error,
             );
-            $prop_value = $obj ? $obj->__ORM_db_value : undef;
+            $prop_value = defined $obj ? $obj->__ORM_db_value : undef;
         }
         elsif( UNIVERSAL::isa( $prop_ref, $class->_prop_class( $prop_name ) ) )
         {
@@ -1441,20 +1426,7 @@ sub _normalize_prop_to_db_value
     return $arg{error}->fatal ? undef : $prop_value;
 }
 
-## use: $self->_validate_prop( prop=>HASH, error=>ORM::Error )
-##
-##     ,
-##       .
-##
-##  'error' - .
-##  'prop'     ,  
-##    ,  .
-##
-##      new  update,
-##        .
-##    _validate_prop  
-##       
-##  $self->SUPER::_validate_prop(...).
+## use: $self->_validate_prop( prop=>HASH, method=>string, error=>ORM::Error )
 ##
 sub _validate_prop {}
 
@@ -1514,6 +1486,7 @@ sub _init
     $class->_default_prefer_lazy_load( $arg{prefer_lazy_load} );
     $class->_emulate_foreign_keys( $arg{emulate_foreign_keys} );
     $class->_default_cache_size( $arg{default_cache_size} );
+    $class->_current_transaction( undef );
 }
 
 ## use: $base_class->_derive
@@ -1748,13 +1721,13 @@ sub _derive
             $derived->{REV_REFS}{ $pclass->{class}.' '.$pclass->{prop} }
                 = [ $pclass->{class}, $pclass->{prop} ];
         }
-        ##          mod_perl,
-        ##       .
+        ## Following pease of code make sence only in mod_perl environment,
+        ## it is necessary to avoid the following problem:
         ##
-        ##     ORM-   ,
-        ##           ,
-        ## ..     _rev_refs   
-        ## httpd.     .
+        ## If you have created and loaded new ORM-class My::Class2 that contain
+        ## referencing property to class My::Class1, then My::Class1 does not
+        ## know about new referer and therefore My::Class1->_rev_refs returns
+        ## outdated data.
         ##
         for my $prop ( keys %{$derived->{TABLE_STRUCT}{$table}} )
         {
@@ -1798,8 +1771,8 @@ sub _values_are_not_equal
 }
 
 ##
-## METHODS AND PROPERTIES TO USE VIA CLASS INITIALISATION
-## (ORM->_derive)
+## METHODS AND PROPERTIES TO USE DURING CLASS INITIALISATION
+## ( ORM->_derive )
 ##
 
 sub _class_is_primary         { ! exists $_[1]->_class_info->{TABLE}; }
@@ -1870,7 +1843,7 @@ sub _load_ORM_class
 
 sub DESTROY
 {
-    $_[0]->_cache && $_[0]->_cache->delete( $_[0] );
+	exists $_[0]->_class_hier->{PRIMARY_CLASS} && $_[0]->_cache && $_[0]->_cache->delete( $_[0] );
 }
 
 1;
