@@ -1,15 +1,29 @@
 #
-# DESCRIPTION:
+# DESCRIPTION
 #   PerlORM - Object relational mapper (ORM) for Perl. PerlORM is Perl
 #   library that implements object-relational mapping. Its features are
 #   much similar to those of Java's Hibernate library, but interface is
 #   much different and easier to use.
 #
-# AUTHOR:
+# AUTHOR
 #   Alexey V. Akimov <akimov_alexey@sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (c) 2005 Alexey V. Akimov. All rights reserved.
+#   Copyright (C) 2005-2006 Alexey V. Akimov
+#
+#   This library is free software; you can redistribute it and/or
+#   modify it under the terms of the GNU Lesser General Public
+#   License as published by the Free Software Foundation; either
+#   version 2.1 of the License, or (at your option) any later version.
+#   
+#   This library is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+#   Lesser General Public License for more details.
+#   
+#   You should have received a copy of the GNU Lesser General Public
+#   License along with this library; if not, write to the Free Software
+#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
 package ORM::Metaprop;
@@ -62,21 +76,19 @@ sub new
 ## )
 ##
 ## prop_class:
-##  ,    .
+##  The class that 'prop' property belongs to.
 ##
 ## prop:
 ## 
-##  prop              =~ ( '->' _ | '-<' _ )+
-##  _ =~ _ '.' _ '.' 
+##  prop              =~ ( '->' DIRECT_PROPERTY | '-<' REVERSE_PROPERTY )+
+##  REVERSE_PROPERTY  =~ REFERRING_CLASS '.' CLASS_PROPERTY '.' ALIAS
 ## 
-##  _   -     
-##  _ -   ,   
-##  _ - ,       
-##                      
-##  _   -      
-##               - - ,   
-##                             
-##                      
+##  DIRECT_PROPERTY   - property of the target class
+##  REVERSE_PROPERTY  - property of third class that refers to target class
+##  REFERRING_CLASS   - class that refers to target class by one of its properties
+##  CLASS_PROPERTY    - property name of the instance of the referring class
+##  ALIAS             - alpha-numeric string, aliasing allows to use different 
+##                      referring objects of the same type
 ##
 sub _new
 {
@@ -91,7 +103,7 @@ sub _new
         $self = $class->_new_flat( class=>$arg{prop_class}, prop=>$prop[0]{name} );
     }
 
-    if( $self )
+    if( defined $self )
     {
         for( my $i=1; $i<@prop; $i++ )
         {
@@ -121,11 +133,11 @@ sub _new
 ## )
 ##
 ## class:
-##  ,    .
+##  The class that 'prop' property belongs to.
 ##
 ## prop:
-##    
-##     ,   id
+##  Direct property of the class.
+##  If this argument is omitted then 'id' is assumed.
 ##
 sub _new_flat
 {
@@ -257,10 +269,9 @@ sub _sql_str
 ## METHODS
 ##
 
-## use: $node->_expand( STRING );
+## use: $metaprop->_expand( STRING );
 ##
-##   prop  ,
-##      .
+## Extends $metaprop meta-property to be 'prop' meta-property of $metaprop.
 ##
 sub _expand
 {
@@ -393,13 +404,12 @@ sub _arb_expand
 
 ## use: @prop = $prop->_parse_prop_str( str=>STRING );
 ##
-##      ,
-##   :
+## Each element of resulting array is hash, containing fields:
 ##
-##   type:  '>' -   '<' -  
-##   name:   
-##   class: (   )  
-##   alias: (   ) 
+##   type:  '>' - direct or '<' - reverse property
+##   name:  name of the property
+##   class: (only makes sence for reverse properties) referring class
+##   alias: (only makes sence for reverse properties) alias
 ##
 sub _parse_prop_str
 {
